/****************************************************************************
 *  Copyright (c) 2015 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - messaging library
 * \file Logging.h
 * \brief
 * Log related classes.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OBLO_MESSENGER_LOGGING_H
#define OBLO_MESSENGER_LOGGING_H

#include "messenger/Common.h"

namespace Messenger {
/**
 * @interface ILogListener
 * @brief Interface which is used to report logging messages to.
 */
class DLLExport ILogListener {
public:
    /**
     * @fn virtual void OnTrace(const std::string& _Msg) = 0;
     * @brief Used to report trace messages. Trace messages have the lowest log level.
     * @param _Msg Log message.
     */
    virtual void OnTrace(const std::string& _Msg) = 0;
    /**
     * @fn virtual void OnDebug(const std::string& _Msg) = 0;
     * @brief Used to report debug messages.
     * @param _Msg Log message.
     */
    virtual void OnDebug(const std::string& _Msg) = 0;
    /**
     * @fn virtual void OnInformation(const std::string& _Msg) = 0;
     * @brief Used to report information messages.
     * @param _Msg Log message.
     */
    virtual void OnInformation(const std::string& _Msg) = 0;
    /**
     * @fn virtual void OnWarning(const std::string& _Msg) = 0;
     * @brief Used to report warning messages.
     * @param _Msg Log message.
     */
    virtual void OnWarning(const std::string& _Msg) = 0;
    /**
     * @fn virtual void OnError(const std::string& _Msg) = 0;
     * @brief Used to report error messages.
     * @param _Msg Log message.
     */
    virtual void OnError(const std::string& _Msg) = 0;
};

}
#endif
