/****************************************************************************
 *  Copyright (c) 2015 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - messaging library
 * \file MqttMessage.h
 * \brief
 * MQTT message classes.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OBLO_MESSENGER_MQTTMESSAGE_H
#define OBLO_MESSENGER_MQTTMESSAGE_H

extern "C" {
#include "paho/MQTTClient.h"
}

#include "Poco/SharedPtr.h"

#include "messenger/Common.h"

namespace Mqtt {
/**
 * @class CWillConfig
 * @brief MQTT Last Will Message configuration.
 */
class DLLExport CMessage {
public:
    typedef Poco::SharedPtr<CMessage> Ptr;
    typedef enum { QOS0=0, QOS1, QOS2 } QosLevel;/**
     * @fn CMessage();
     * @brief Default constructor.
     */
    CMessage();
    /**
     * @fn CMessage(const std::string& _szPayload);
     * @brief CMessage
     * @param _szPayload
     */
    CMessage(const std::string& _szPayload);
    /**
     * @brief Copy constructor.
     * @brief CMessage
     * @param _Msg
     */
    CMessage(const CMessage& _Msg);
    /**
     * @fn virtual ~CMessage();
     * @brief Destructor.
     */
    virtual ~CMessage();
    /**
     * @fn CMessage& operator=(const CMessage& _Msg);
     * @brief Aassignment operator.
     */
    CMessage& operator=(const CMessage& _Msg);
    /**
     * @fn std::string GetPayload() const;
     * @brief GetPayload
     * @return
     */
    std::string GetPayload() const;
    /**
     * @fn std::string GetPayloadHolder() const;
     * @brief GetPayload
     * @return
     */
    std::string& GetPayloadHolder();
    /**
     * @fn void SetPayload(const std::string& _szPayload);
     * @brief SetPayload
     * @param _szPayload
     */
    void SetPayload(const std::string& _szPayload);
    /**
     * @fn QosLevel GetQos() const;
     * @brief GetQos
     * @return
     */
    QosLevel GetQos() const;
    /**
     * @fn void SetQos(QosLevel _Level);
     * @brief SetQos
     * @param _Level
     */
    void SetQos(QosLevel _Level);
    /**
     * @fn bool IsDuplicated() const;
     * @brief IsDuplicated
     * @return
     */
    bool IsDuplicated() const;
    /**
     * @fn void SetDuplicated(bool _bDuplicated);
     * @brief SetDuplicated
     * @param _bDuplicated
     */
    void SetDuplicated(bool _bDuplicated);
    /**
     * @fn bool IsRetained() const;
     * @brief IsRetained
     * @return
     */
    bool IsRetained() const;
    /**
     * @fn void SetRetained(bool _bRetained);
     * @brief SetRetained
     * @param _bRetained
     */
    void SetRetained(bool _bRetained);
    /**
     * @fn void WriteMessage(MQTTClient_message** _pMsg) const;
     * @brief Writes a message to a Paho MQTT message.
     * @param _pMsg
     */
    void WriteMessage(MQTTClient_message** _pMsg) const;
    /**
     * @fn static void FreeMessage(MQTTClient_message** _pMsg);
     * @brief Dealocates memory that is allocated for a MQTT message.
     * @param _pMsg
     */
    static void FreeMessage(MQTTClient_message** _pMsg);
    /**
     * @fn static CMessage::Ptr ParseMessage(const MQTTClient_message* _pMsg);
     * @brief Parses Paho message to corresponding format.
     * @param _pMsg Message to parse.
     * @return Formated message.
     */
    static CMessage::Ptr ParseMessage(const MQTTClient_message* _pMsg);
private:
    std::string m_szPayload;
    QosLevel m_QoS;
    bool m_bRetained;
    bool m_bDuplicated;
};

}

#endif
