/****************************************************************************
 *  Copyright (c) 2015 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - messaging library
 * \file Publishing.h
 * \brief
 * Publishing classes.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OBLO_MESSENGER_PUBLISHING_H
#define OBLO_MESSENGER_PUBLISHING_H

#include "messenger/Common.h"
#include "messenger/Runners.h"
#include "messenger/MqttMessage.h"

#include <queue>

namespace Mqtt {
/**
 * @class CPublishHolder
 * @brief Publish message holder.
 */
class DLLExport CPublishHolder {
public:
    typedef Poco::SharedPtr<CPublishHolder> Ptr;
    /**
     * @fn CPublishHolder(const std::string& _szTopic, const Mqtt::CMessage::Ptr _pMsg);
     * @brief Constructor.
     * @param _szTopic
     * @param _pMsg
     */
    CPublishHolder(const std::string& _szTopic, const Mqtt::CMessage::Ptr _pMsg);
    /**
     * @fn const std::string GetTopic() const;
     * @brief GetTopic
     * @return
     */
    const std::string GetTopic() const;
    /**
     * @fn const Mqtt::CMessage::Ptr GetMessage() const;
     * @brief GetMessage
     * @return
     */
    const Mqtt::CMessage::Ptr GetMessage() const;
private:
    const std::string m_szTopic;
    const Mqtt::CMessage::Ptr m_pMsg;
};
/**
 * @class CRecvHolder
 * @brief Holder for a received message.
 */
class DLLExport CRecvHolder {
public:
    typedef Poco::SharedPtr<CRecvHolder> Ptr;
    /**
     * @fn CRecvHolder(const std::string& _szTopic, const Mqtt::CMessage::Ptr _pMsg);
     * @brief CRecvHolder
     * @param _szTopic
     * @param _pMsg
     */
    CRecvHolder(const std::string& _szTopic, const Mqtt::CMessage::Ptr _pMsg);
    /**
     * @fn const std::string GetTopic() const;
     * @brief GetTopic
     * @return
     */
    const std::string GetTopic() const;
    /**
     * @fn const Mqtt::CMessage::Ptr GetMessage() const;
     * @brief GetMessage
     * @return
     */
    const Mqtt::CMessage::Ptr GetMessage() const;
private:
    const std::string m_szTopic;
    const Mqtt::CMessage::Ptr m_pMsg;
};

}

#endif
